@@ -1,6 +1,7 @@
 from dexy.constants import Constants
 from dexy.version import Version
 from ordereddict import OrderedDict
+import codecs
 import dexy.introspect
 import json
 import glob
@@ -17,6 +18,8 @@ import zlib
 
 class Artifact(object):
     HASH_WHITELIST = Constants.ARTIFACT_HASH_WHITELIST
+    MAX_DATA_DICT_DECIMALS = 5
+    MAX_DATA_DICT_LENGTH = 10 ** MAX_DATA_DICT_DECIMALS
     META_ATTRS = [
         'additional',
         'binary_input',
@@ -47,6 +50,8 @@ class Artifact(object):
         '.eot',
         '.ttf',
         '.woff',
+        '.sqlite',
+        '.sqlite3',
         '.swf'
     ]
 
@@ -94,7 +99,7 @@ class Artifact(object):
         return str(self.state) == 'complete'
 
     @classmethod
-    def retrieve(klass, hashstring):
+    def retrieve(klass, hashstring, hashfunction='md5'):
         if not hasattr(klass, 'retrieved_artifacts'):
             klass.retrieved_artifacts = {}
         if klass.retrieved_artifacts.has_key(hashstring):
@@ -102,6 +107,7 @@ class Artifact(object):
         else:
             artifact = klass()
             artifact.hashstring = hashstring
+            artifact.hashfunction = hashfunction
             artifact.load()
             klass.retrieved_artifacts[hashstring] = artifact
             return artifact
@@ -302,7 +308,7 @@ class Artifact(object):
             h = hashlib.sha512()
 
             if self.data_dict and len(self.data_dict) > 0:
-                h.update(self.output_text())
+                h.update(self.output_text().encode("utf-8"))
 
             elif self.is_canonical_output_cached:
                 self.state = 'complete'
@@ -383,21 +389,54 @@ class Artifact(object):
         self.data_dict['1'] = data
 
     def set_data_from_artifact(self):
-        f = open(self.filepath())
+        f = codecs.open(self.filepath(), "r", encoding="utf-8")
         self.data_dict['1'] = f.read()
 
     def is_loaded(self):
         return hasattr(self, 'data_dict') and len(self.data_dict) > 0
 
     def compute_hash(self, text):
+        unicode_text = None
+
+        if type(text) == unicode:
+            unicode_text = text
+        elif type(text) == dict:
+            unicode_text = json.dumps(text)
+        elif self.binary_input:
+            pass
+        else:
+            unicode_text = unicode(text, encoding="utf-8")
+
+        if unicode_text:
+            text = unicode_text.encode("utf-8")
+
         if self.hashfunction == 'md5':
-            h = hashlib.md5(str(text)).hexdigest()
+            h = hashlib.md5(text).hexdigest()
+
+        elif self.hashfunction == 'sha1':
+            h = hashlib.sha1(text).hexdigest()
+
+        elif self.hashfunction == 'sha224':
+            h = hashlib.sha224(text).hexdigest()
+
+        elif self.hashfunction == 'sha256':
+            h = hashlib.sha256(text).hexdigest()
+
+        elif self.hashfunction == 'sha384':
+            h = hashlib.sha384(text).hexdigest()
+
+        elif self.hashfunction == 'sha512':
+            h = hashlib.sha512(text).hexdigest()
+
         elif self.hashfunction == 'crc32':
-            h =str(zlib.crc32(str(text)) & 0xffffffff )
+            h = str(zlib.crc32(text) & 0xffffffff)
+
         elif self.hashfunction == 'adler32':
-            h =str(zlib.adler32(str(text)) & 0xffffffff )
+            h = str(zlib.adler32(text) & 0xffffffff)
+
         else:
             raise Exception("unexpected hash function %s" % self.hashfunction)
+
         return h
 
     def input_hashes(self):
@@ -464,11 +503,18 @@ class Artifact(object):
         except IOError as e:
             self.save_meta()
 
+
+    def convert_if_not_unicode(self, s):
+        if type(s) == unicode:
+            return s
+        else:
+            return unicode(s, encoding="utf-8")
+
     def input_text(self):
-        return "".join([v for k, v in self.input_data_dict.items()])
+        return u"".join([self.convert_if_not_unicode(v) for k, v in self.input_data_dict.items()])
 
     def output_text(self):
-        return "".join([v for k, v in self.data_dict.items()])
+        return u"".join([self.convert_if_not_unicode(v) for k, v in self.data_dict.items()])
 
     def relative_refs(self, relative_to_file):
         """How to refer to this artifact, relative to another."""
@@ -501,7 +547,7 @@ class Artifact(object):
         if not work_filename:
             work_filename = self.work_filename()
         work_path = os.path.join(self.artifacts_dir, work_filename)
-        work_file = open(work_path, "w")
+        work_file = codecs.open(work_path, "w", encoding="utf-8")
         work_file.write(self.input_text())
         work_file.close()
 
@@ -510,7 +556,7 @@ class Artifact(object):
 
     def open_tempfile(self, ext):
         tempfile_path = os.path.join(self.artifacts_dir, self.temp_filename(ext))
-        open(tempfile_path, "w")
+        codecs.open(tempfile_path, "w", encoding="utf-8")
 
     def temp_dir(self):
         return os.path.join(self.artifacts_dir, self.hashstring)
@@ -524,8 +570,11 @@ class Artifact(object):
             # write all inputs to this directory, under their canonical names
             for input_artifact in self._inputs.values():
                 filename = os.path.join(tempdir, input_artifact.canonical_filename())
-                input_artifact.write_to_file(filename)
-                self.log.debug("Populating temp dir for %s with %s" % (self.key, filename))
+                if os.path.exists(input_artifact.filepath()):
+                    input_artifact.write_to_file(filename)
+                    self.log.debug("Populating temp dir for %s with %s" % (self.key, filename))
+                else:
+                    self.log.warn("Skipping file %s for temp dir for %s, file does not exist (yet)" % (filename, self.key))
 
             # write the workfile to this directory under its canonical name
             previous = self.previous_artifact_filepath
@@ -644,3 +693,23 @@ class Artifact(object):
         relpath = self.relative_path_to_input(input_artifact)
         return os.path.join(relpath, os.path.basename(input_artifact.key))
 
+    def convert_numbered_dict_to_ordered_dict(self, numbered_dict):
+        ordered_dict = OrderedDict()
+        for x in sorted(numbered_dict.keys()):
+            k = x.split(":", 1)[1]
+            ordered_dict[k] = numbered_dict[x]
+        return ordered_dict
+
+    def convert_data_dict_to_numbered_dict(self):
+        if len(self.data_dict) >= self.MAX_DATA_DICT_LENGTH:
+            exception_msg = """Your data dict has %s items, which is greater than the arbitrary limit of %s items.
+            You can increase this limit by changing MAX_DATA_DICT_DECIMALS."""
+            raise Exception(exception_msg % (len(self.data_dict), self.MAX_DATA_DICT_LENGTH))
+
+        data_dict = {}
+        i = -1
+        for k, v in self.data_dict.iteritems():
+            i += 1
+            fmt = "%%0%sd:%%s" % self.MAX_DATA_DICT_DECIMALS
+            data_dict[fmt % (i, k)] = v
+        return data_dict
