@@ -1,11 +1,16 @@
 from dexy.constants import Constants
 from dexy.controller import Controller
+from dexy.utils import get_log
 from dexy.version import Version
 from modargs import args
-import copy
+from pygments import highlight
+from pygments.lexers.agile import PythonLexer
+from pygments.formatters import TerminalFormatter
 import cProfile
+import copy
 import datetime
 import dexy.introspect
+import inspect
 import os
 import shutil
 import sys
@@ -14,10 +19,21 @@ MOD = sys.modules[__name__]
 PROG = 'dexy'
 S = "   "
 
+# List of filter classes not to include in the filters command
+NODOC_FILTERS = [
+    "SubprocessCompileFilter",
+    "SubprocessCompileInputFilter",
+    "SubprocessFilter",
+    "SubprocessStdoutFilter",
+    "SubprocessStdoutInputFileFilter",
+    "SubprocessStdoutInputFilter"
+    ]
+
 def run():
     args.parse_and_run_command(sys.argv[1:], MOD, default_command=Constants.DEFAULT_COMMAND)
 
 def dexy_command(
+        allreports=False, # whether to run all available reports
         artifactclass=Constants.DEFAULT_ACLASS, # name of class to use for artifacts
         artifactsdir=Constants.DEFAULT_ADIR, # location of directory in which to store artifacts
         config=Constants.DEFAULT_CONFIG, # name to use for configuration file
@@ -33,7 +49,6 @@ def dexy_command(
         help=False, # DEPRECATED just to catch people who use the old dexy --help syntax
         hashfunction='md5', # What hash function to use, set to crc32 or adler32 for more speed, less reliability
         ignore=False, # whether to ignore nonzero exit status or raise an error - may not be supported by all filters
-        local=False, # use cached local copies of remote URLs, faster but might not be up to date
         logfile=Constants.DEFAULT_LFILE, # name of log file
         logsdir=Constants.DEFAULT_LDIR, # location of directory in which to store logs
         nocache=False, # whether to force artifacts to run even if there is a matching file in the cache
@@ -42,8 +57,10 @@ def dexy_command(
         reporters=False, # DEPRECATED just to catch people who use the old dexy --reporters syntax
         reports=Constants.DEFAULT_REPORTS, # reports to be run after dexy runs, enclose in quotes and separate with spaces
         reset=False, # whether to purge existing artifacts and logs before running Dexy
+        run="", # specific document to run. if specified, this document + its dependencies will be all that is run
         setup=False, # DEPRECATED just to catch people who use the old dexy --setup syntax
         strictinherit=False, # set to true if you want 'allinputs' to only reference items in same dir or a subdir
+        uselocals=True, # use cached local copies of remote URLs, faster but might not be up to date, 304 from server will override this setting
         version=False # DEPRECATED just to catch people who use the old dexy --version syntax
     ):
     """
@@ -76,8 +93,13 @@ def dexy_command(
 
     If you run into trouble, visit http://dexy.it/help
     """
+    # catch deprecated arguments
+    if help or version or reporters or filters:
+        print "the command syntax has changed, please type 'dexy help' for help"
+        sys.exit(1)
+
     ### @export "dexy-command-check-setup"
-    if not check_setup():
+    if not check_setup(logsdir=logsdir, artifactsdir=artifactsdir):
         print "Please run '%s setup' first to create the directories dexy needs to work with" % PROG
         sys.exit(1)
 
@@ -88,16 +110,33 @@ def dexy_command(
     if output:
         if not reports == Constants.DEFAULT_REPORTS:
             raise Exception("if you pass --output you can't also modify reports! pick 1!")
+        if allreports:
+            raise Exception("if you pass --output you can't also pass --allreports!")
         reports = "Output"
 
-    # catch deprecated arguments
-    if help or version or reporters or filters:
-        print "the command syntax has changed, please type 'dexy help' for help"
-        sys.exit(1)
+    if allreports:
+        if not reports == Constants.DEFAULT_REPORTS:
+            raise Exception("if you pass --allreports you can't also specify --reports")
 
     controller = run_dexy(locals())
     if not dryrun:
-        reports_command(reports=reports, logsdir=logsdir, controller=controller)
+        if allreports:
+            reports_command(
+                allreports=True,
+                artifactclass=artifactclass,
+                controller=controller,
+                hashfunction=hashfunction,
+                logsdir=logsdir
+            )
+        else:
+            reports_command(
+                reports=reports,
+                artifactclass=artifactclass,
+                controller=controller,
+                hashfunction=hashfunction,
+                logsdir=logsdir
+            )
+
 
 def run_dexy(args):
     # validate args and do any conversions required
@@ -147,7 +186,7 @@ def setup_command(logsdir=Constants.DEFAULT_LDIR, artifactsdir=Constants.DEFAULT
     directories and they aren't created automatically as a precaution against
     accidentally running dexy somewhere you didn't mean to.
     """
-    if check_setup():
+    if check_setup(logsdir=logsdir, artifactsdir=artifactsdir):
         print "Dexy is already set up. Run 'reset' if you want to reset everything."
     else:
         if not os.path.exists(logsdir):
@@ -267,25 +306,115 @@ def version_command():
     """Print the current version."""
     print "%s version %s" % (PROG, Version.VERSION)
 
-def filters_command():
-    """Lists currently available dexy filters."""
-    print filters_text()
+def filters_command(
+        alias="", # If a filter alias is specified, more detailed help for that filter is printed.
+        nocolor=False, # When source = True, whether to omit syntax highlighting
+        showall=False, # Whether to show all filters, including those which need missing software, implies versions=True
+        showmissing=False, # Whether to just show filters missing external software, implies versions=True
+        space=False, # Whether to add extra spacing to the output for extra readability
+        source=False, # Whether to include syntax-highlighted source code when displaying an indvidual filter
+        versions=False # Whether to check the installed version of external software required by filters, slower
+    ):
+    """
+    Lists the available dexy filters and their aliases.
 
-def filters_text():
-    filters = dexy.introspect.filters()
-    text = []
-    for k in sorted(filters.keys()):
-        klass = filters[k]
-        version = klass.version()
-        if (version is None) or version:
-            text.append("\n%s : %s" % (k, klass.__name__))
-            if klass.executable():
-                text.append(S + "calls '%s'" % klass.executable())
-                if version:
-                    text.append(S + version + " detected")
-            if klass.__doc__:
-                text.append(S + klass.__doc__.strip())
-    return "\n".join(text) + "\n"
+    Does not include filters which require python modules that are not
+    installed.
+
+    Consult the Dexy website http://dexy.it for a complete list of filters.
+    """
+    if len(alias) == 0:
+        # This tends to be slow, let people know it's running
+        print "looking up filter information..."
+    else:
+        if showall:
+            raise Exception("can't specify an alias if showall is True")
+
+    if check_setup():
+        log = get_log()
+    else:
+        log = Constants.NULL_LOGGER
+
+    print filters_text(alias, nocolor, showall, showmissing, space, source, versions, log)
+
+def filters_text(
+        alias="", # If a filter alias is specified, more detailed help for that filter is printed.
+        nocolor=False, # When source = True, whether to omit syntax highlighting
+        showall=False, # Whether to show all filters, including those which need missing software, implies versions=True
+        showmissing=False, # Whether to just show filters missing external software, implies versions=True
+        space=False, # Whether to add extra spacing to the output for extra readability
+        source=False, # Whether to include syntax-highlighted source code when displaying an indvidual filter
+        versions=False, # Whether to check the installed version of external software required by filters, slower
+        log=Constants.NULL_LOGGER
+        ):
+    filters_dict = dexy.introspect.filters(log)
+
+    if len(alias) > 0:
+        # We want help on a particular filter
+        klass = filters_dict[alias]
+        text = []
+        text.append(klass.__name__)
+        text.append("")
+        text.append("Aliases: %s" % ", ".join(klass.ALIASES))
+        text.append("")
+        text.append(trim_docstring(klass.__doc__))
+        text.append("")
+        text.append("http://dexy.it/docs/filters/%s" % alias)
+        if source:
+            text.append("")
+            source_code = inspect.getsource(klass)
+            if nocolor:
+                text.append(source_code)
+            else:
+                formatter = TerminalFormatter()
+                lexer = PythonLexer()
+                text.append(highlight(source_code, lexer, formatter))
+        return "\n".join(text)
+
+    else:
+        def sort_key(k):
+            return k.__name__
+
+        filter_classes = sorted(set(f for f in filters_dict.values()), key=sort_key) # uniqify and sort class names
+
+        text = []
+        for klass in filter_classes:
+            if not showall:
+                skip = klass.__name__ in NODOC_FILTERS
+            else:
+                skip = False
+
+            if (versions or showmissing or showall) and not skip:
+                version = klass.version()
+                no_version_info_available = (version is None)
+                if no_version_info_available:
+                    version_message = ""
+                    if showmissing:
+                        skip = True
+                elif version:
+                    version_message = "Installed version: %s" % version
+                    if showmissing:
+                        skip = True
+                else:
+                    if not (showmissing or showall):
+                        skip = True
+                    version_message = "'%s' failed, filter may not be available." % klass.version_command()
+
+            if not skip:
+                name_and_aliases = "%s (%s) " % (klass.__name__, ", ".join(klass.ALIASES))
+                docstring = trim_docstring(klass.__doc__)
+                if "\n" in docstring:
+                    docstring = docstring.splitlines()[0]
+                filter_help = name_and_aliases + docstring
+                if (versions or showmissing or (showall and not version)):
+                    filter_help += " %s" % version_message
+                text.append(filter_help)
+
+        if space:
+            sep = "\n\n"
+        else:
+            sep = "\n"
+        return sep.join(text)
 
 def reporters_command():
     """Lists currently available dexy reporters."""
@@ -318,10 +447,11 @@ def report_command(**kwargs):
     reports_command(**kwargs)
 
 def reports_command(
-        artifactclass=Constants.DEFAULT_ACLASS, # What artifact class to use (must correspond to artifact class used by batch id)
         allreports=False, # whether to run all available reporters (except those that are specifically excluded by setting ALLREPORTS=False)
-        controller=False, # can pass the just-run controller instance to save having to load some data from disk which is already in memory
+        artifactclass=Constants.DEFAULT_ACLASS, # What artifact class to use (must correspond to artifact class used by batch id)
         batchid=False, # What batch id to run reports for. Leave false to run the most recent batch available (recommended).
+        controller=False, # can pass the just-run controller instance to save having to load some data from disk which is already in memory
+        hashfunction='md5',
         logsdir=Constants.DEFAULT_LDIR, # The location of the logs directory.
         reports="OutputReporter RunReporter LongOutputReporter OutputTgzReporter" # The class names for all reports to be run.
     ):
@@ -343,6 +473,7 @@ def reports_command(
         reports = []
     reporters = dexy.introspect.reporters()
     if allreports:
+        print "using allreports"
         reports = [k for k, v in reporters.items() if v.ALLREPORTS]
 
     for r in reports:
@@ -356,6 +487,7 @@ def reports_command(
                 artifact_class=artifactclass,
                 batch_id=batchid,
                 controller=controller,
+                hashfunction=hashfunction,
                 logsdir=logsdir
             )
 
@@ -365,3 +497,28 @@ def reports_command(
 def it_command(**kwargs):
     dexy_command(kwargs)
 
+# From http://www.python.org/dev/peps/pep-0257/#handling-docstring-indentation
+def trim_docstring(docstring):
+    if not docstring:
+        return ''
+    # Convert tabs to spaces (following the normal Python rules)
+    # and split into a list of lines:
+    lines = docstring.expandtabs().splitlines()
+    # Determine minimum indentation (first line doesn't count):
+    indent = sys.maxint
+    for line in lines[1:]:
+        stripped = line.lstrip()
+        if stripped:
+            indent = min(indent, len(line) - len(stripped))
+    # Remove indentation (first line is special):
+    trimmed = [lines[0].strip()]
+    if indent < sys.maxint:
+        for line in lines[1:]:
+            trimmed.append(line[indent:].rstrip())
+    # Strip off trailing and leading blank lines:
+    while trimmed and not trimmed[-1]:
+        trimmed.pop()
+    while trimmed and not trimmed[0]:
+        trimmed.pop(0)
+    # Return a single string:
+    return '\n'.join(trimmed)
