@@ -99,9 +99,10 @@ class SubprocessFilter(ProcessFilter):
         self.artifact.stdout = stdout
 
     def run_command(self, command, env, input_text = None):
-        self.log.debug("about to run '%s'" % command)
+        cwd = self.setup_cwd()
+        self.log.debug("about to run '%s' in %s" % (command, cwd))
         proc = subprocess.Popen(command, shell=True,
-                                cwd=self.setup_cwd(),
+                                cwd=cwd,
                                 stdin=subprocess.PIPE,
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.STDOUT,
@@ -116,6 +117,7 @@ class SubprocessStdoutFilter(SubprocessFilter):
 
     def process(self):
         command = self.command_string_stdout()
+        self.log.debug("About to run '%s'" % command)
         proc, stdout = self.run_command(command, self.setup_env())
         self.handle_subprocess_proc_return(proc.returncode, stdout)
         self.artifact.set_data(stdout)
