@@ -2,13 +2,14 @@ from dexy.dexy_filter import DexyFilter
 from ordereddict import OrderedDict
 from pygments import highlight
 from pygments.formatters import get_formatter_for_filename
-from pygments.lexers import get_lexer_for_filename
 from pygments.lexers import get_lexer_by_name
+from pygments.lexers import get_lexer_for_filename
 from pygments.lexers.agile import PythonConsoleLexer
 from pygments.lexers.agile import RubyConsoleLexer
-from pygments.lexers.special import TextLexer
+from pygments.lexers.text import TexLexer
 from pygments.lexers.web import JavascriptLexer
 from pygments.lexers.web import XmlLexer
+import pygments.lexers.web
 
 class PygmentsFilter(DexyFilter):
     """
@@ -37,10 +38,13 @@ class PygmentsFilter(DexyFilter):
                 lexer = PythonConsoleLexer()
             elif ext == '.rbcon':
                 lexer = RubyConsoleLexer()
-            elif ext in ('.json', '.dexy'):
+            elif (ext in ('.json', '.dexy') or self.artifact.name.endswith(".dexy")) and (pygments.__version__ < '1.5'):
                 lexer = JavascriptLexer()
+            elif ext in ('.dexy') or self.artifact.name.endswith(".dexy"):
+                # JSON lexer available in pygments 1.5
+                lexer = pygments.lexers.web.JSONLexer()
             elif ext == '.Rd':
-                lexer = TextLexer()
+                lexer = TexLexer() # does a passable job
             elif ext == '.svg':
                 lexer = XmlLexer()
             else:
@@ -60,17 +64,12 @@ class PygmentsFilter(DexyFilter):
         if self.artifact.ext in self.IMAGE_OUTPUT_EXTENSIONS:
             self.artifact.binary_output = True
             # TODO set to final
-            f = open(self.artifact.filepath(), 'w')
-            f.write(highlight(self.artifact.input_text(), lexer, formatter))
-            f.close()
+            with open(self.artifact.filepath(), 'wb') as f:
+                f.write(highlight(self.artifact.input_text(), lexer, formatter))
         else:
             output_dict = OrderedDict()
             for k, v in input_dict.items():
                 # TODO figure out where these characters are coming from and don't hard-code this.
-                v = str(v.replace(" \x08", "").replace(chr(13), ""))
-                try:
-                    output_dict[k] = str(highlight(v, lexer, formatter))
-                except UnicodeEncodeError as e:
-                    self.artifact.log.warn("error processing section %s of file %s" % (k, self.artifact.key))
-                    raise e
+                v = v.replace(" \x08", "").replace(chr(13), "")
+                output_dict[k] = highlight(v, lexer, formatter)
             return output_dict
